from datetime import datetime, timedelta
import re

def srt_parser(file, text, shift, *kwargs):
    """
    Parser for SubRip (.srt) and WebVTT (.vtt) files.
    """
    for line in text:
        if '-->' in line:
            # remove leading and trailing spaces
            line = line.strip()
            # force period as decimal separator
            line = line.replace(',', '.')
            # extract time stamps
            start, end = line.split('-->')
            start = start.strip()
            end = end.strip()
            # shift
            start_dt = datetime.strptime(start, '%H:%M:%S.%f') + timedelta(seconds=shift)
            end_dt = datetime.strptime(end, '%H:%M:%S.%f') + timedelta(seconds=shift)
            start = start_dt.strftime('%H:%M:%S.%f')[:-3]
            end = end_dt.strftime('%H:%M:%S.%f')[:-3]
            line = f'{start} --> {end}\n'
        # write line
        file.write(line)

def sub_parser(file, text, shift, framerate=24):
    """
    Parser for MicroDVD (.sub) files.
    """
    for line in text:
        regexp = re.search('^\{(\d+)\}\{(\d+)\}(.+)$', line)
        if regexp:
            # decompose line
            start = int(regexp.group(1))
            end = int(regexp.group(2))
            content = regexp.group(3)
            # shift
            start = start + round(shift * framerate)
            end = end + round(shift * framerate)
            line = f'{{{start}}}{{{end}}}{content}\n'
        # write line
        file.write(line)        


# Mapping for the various supported formats
FORMATS = {'srt': srt_parser,
           'vtt': srt_parser,
           'sub': sub_parser}


from setuptools import setup

with open('README.md', 'r') as fh:
    readme = fh.read()

with open('submv/_version.py') as f:
    exec(f.read())
    
args = dict(name='submv',
            version=__version__,
            description='Subtitle shift for resynchronization',
            long_description=readme,
            long_description_content_type='text/markdown',
            author='Joris Paret',
            author_email='joris.paret@gmail.com',
            maintainer='Joris Paret',
            url='https://github.com/jorisparet/submv',
            keywords=['subtitle', 'sub', 'srt', 'shift',
                      'move', 'synchronization', 'sync'],
            entry_points={'console_scripts':
                          ['submv = submv.main:main']},
            license='GPLv3',
            classifiers=[
                'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
                'Development Status :: 3 - Alpha',
                'Topic :: Utilities ',
                'Programming Language :: Python :: 3',
                'Operating System :: POSIX :: Linux',
                'Operating System :: Microsoft :: Windows',
                'Intended Audience :: End Users/Desktop',
                'Natural Language :: English']
)

setup(**args)



#!/usr/bin/env python

import argparse
from .formats import FORMATS

ENCODINGS = ['utf_8', 'iso8859_1', 'mac_roman']
formats_listed = list(FORMATS.keys())

def main():

    # Parser
    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    description = f"""{parser.prog} shifts the timecodes of a subtitle file
    by a given amount to synchronize it to a video stream.""".replace('\n',' ')
    parser.description = description
    parser.add_argument('input',
                        type=str, 
                        help='path to the input file')
    parser.add_argument('shift',
                        type=float,
                        help='shift (in seconds)')
    parser.add_argument('-o', '--output',
                        type=str,
                        help='path to the output file. If not provided, it overwrites the input file')
    parser.add_argument('-f', '--format',
                        type=str,
                        default='srt',
                        help=f'input subtitle file format, should be one of {formats_listed}')
    parser.add_argument('-r', '--framerate',
                        type=float,
                        default=24.0,
                        help='video framerate (only affects certain formats)')
    args = parser.parse_args()

    # Custom variables linked to parser
    input_file = args.input
    shift = args.shift
    output_file = args.output
    format = args.format
    framerate = args.framerate

    # Sanity check on the format
    if format not in formats_listed:
        raise ValueError(f'{format} is not a valid or supported file format.')
    sub_parser = FORMATS[format]

    # Read input file with the correct encoding
    input_encoding = None
    text = None
    for e in ENCODINGS:
        try:
            with open(input_file, 'r', encoding=e) as sub_file:
                text = sub_file.readlines()
                input_encoding = e
                break
        except UnicodeDecodeError:
            pass
        except:
            print('An error occured while reading the input file.')

    # Name of the output file
    if output_file is None:
        output_file = input_file

    # Write shifted file
    with open(output_file, 'w', encoding=input_encoding) as sub_file:
        sub_parser(sub_file, text, shift, framerate=framerate)

if __name__ == '__main__':
    main()